'MMEDIT!!! Basic Version = CMM2
'MMEDIT!!! Port = :10,300
'MMEDIT!!! Device = CMM2

'  ===========================================================
'  GEOTHERMAL CONTROL PROGRAM        GCP.BAS V1.00g 06/20/2022 
'             BY: PAUL LEPKOWSKI & LANCE BENSON
'  ===========================================================
'        RUNS ON A PC ON TOP OF WINDOWS
'  ===========================================================

TIMER=0
OPTION EXPLICIT
OPTION BASE 1

' GLOBAL VARIABLES
DIM task%=0,otask%=0 'task, old task, MENU,
dim string a,b,c,d,ch,s,sDT,INP$,USRIN$,sLastSec,sCmd1,sCmd2,sCmd3,sLine(50) length 80
dim string QT=chr$(34)
dim integer i,j,k,l,m,n,ln,fh,nLines,nLoops, mode=9, nSensors=10
dim float f

' ARRAYS AND ELEMENT CONSTANTS
' SENSOR ARRAY SN!( data, pin) ' loads pins--invalid readings, invalid pin#s
DIM SN!(10,2) = ( 999,899,799,699,599,499,399,299,199,99,9,10,11,12,14,15,29,31,32,34) 
CONST Tout =1  'outside
CONST Tin  =2  'inside
CONST Tydf =3  'yard feed
CONST Tydr =4  'yard return
CONST Ttkf =5  'tank feed
CONST Ttkr =6  'tank return
CONST Tctr =7  'tank center
CONST Tbwf =8  'blower feed
CONST Tbwr =9  'blower return
CONST Tduc =10 'duct
CONST Tpin =1  'pin
CONST Tdata=2  'temp data

' RELAY ARRAY (pin, on/off)
DIM RL%(6,2) = ( 0,0,0,0,0,0,0,0,0,0,0,0) ' invalid pin#s all off
CONST Rfc  =1 'compressor
CONST Rfr  =2 'reverse valve
CONST Ryl  =3 'yard pump, tank pump
CONST Rbl  =4 'blower pump
CONST Rbw  =5 'blower
CONST Rga  =6 'gas heat
CONST Rpin =1 'pin
CONST Rdata=2 'on/off

INITIALIZE ' set up for main loop

  DO 'begin main loop
    inc nLoops
    DISPLAY.SCREEN
    USRIN$=_input$()
    if USRIN$<>"" then
      text 0,mm.fontheight*29,USRIN$       
      ' process user input
    endif
    if mode <> 0 then 
      sensor.read
      SET.OUT.PINS
    endif
    pause 1000
    do while sLastSec=mid$(time$,7,2): loop: sLastSec=mid$(time$,7,2) ' at least 1 second per loop
  LOOP 'end of main loop

function _input$() ' returns user input when terminated with CR--chr$(13)
  ch=inkey$
  if ch <> "" then
    if ch=chr$(13) then ' carriage return
      _input$=INP$
    else
      if ch = chr$(10) then ' throw away linefeed
      else
        INP$ = INP$+ch
        text 0,mm.fontheight*30,INP$
      endif
    endif
    _input$=""
  endif
end function ' _input$

SUB DETERMINE.ACTION
'  SENDS MMbasic COMMANDS TO A REMOTE PicoMite
'  causing the PicoMite to
'  read the following temperature data using the variables:
'    1 outside air                 Tout
'    2 inside air                  Tin
'    3 yard loop feed              Tydf
'    4 yard loop return            Tydr
'    5 tank loop feed              Ttkf
'    6 blower loop feed            Tblf
'    7 tank loop return            Ttkr
'    8 blower loop return          Tblr
'    9 duct temperature            Tdu
'  control relays which power the following using the variables:
'    1 freon compressor                   Pfc
'    2 freon compressor reversing valve   Pfr
'    3 yard loop pump                     Pyl
'    4 tank loop pump                     Ptl
'    5 blower loop pump                   Pbl
'    6 blower                             Pb
'    7 furnace gas burner                 Pga
'  report temperature data to the CONTROL PROGRAM as follows using the variable:
'    1 outside air                        Tout
'    2 inside air                         Tin
'    3 yard loop feed                     Tydr
'    4 yard loop return                   Tydr
'    5 tank loop feed                     Ttkf
'    6 tank loop center                   Ttkc
'    7 tank loop return                   Ttkr
'    8 blower loop feed                   Tblf
'    9 blower loop return                 Tblr
'   10 duct temperature                   Tdu
'  report on/off status to the CONTROL PROGRAM as follows using the variables:
'    1 heat pump compressor               Pfc
'    2 heat pump reversing valve          Pfr
'    3 yard loop pump                     Pyl
'    4 tank loop pump                     Ptl
'    5 blower loop pump                   Pbl
'    6 blower                             Pb
'    7 furnace gas burner                 Pa
END SUB 'DETERMINE.ACTION

SUB SET.OUT.PINS
END SUB 'SET.OUT.PINS

SUB DISPLAY.SCREEN
'  DIM L$(8) LENGTH 7 = ("├─ ─ ─┤","╞════<╡","╞═══<═╡","╞══<══╡","╞═<═══╡","╞<════╡")
'  DIM Lr%=0, Lc%=0, 
'  DIM R$(8) LENGTH 7 = ("├─ ─ ─┤","╞>════╡","╞═>═══╡","╞══>══╡","╞═══>═╡","╞════>╡")
'  DIM Rr%=0, Rc%=0, Rr%=0, Rc%=0
'  R$(1)="├─ ─ ─┤":R$(2)="╞>════╡":R$(3)="╞═>═══╡":R$(4)="╞══>══╡":R$(5)="╞═══>═╡":R$(6)="╞════>╡"
'  L$(1)="├─ ─ ─┤":L$(2)="╞════<╡":L$(3)="╞═══<═╡":L$(4)="╞══<══╡":L$(5)="╞═<═══╡":L$(6)="╞<════╡"

'  DISPLAY.MENU
'  cls
  text 0,0,"" ' set cursor
  text mm.hres-(mm.fontwidth*7),0,str$(nLoops)
  text 10*mm.fontwidth,1*mm.fontheight, str$(SN!(1,1),3,1) ' outside temperature
  text 25*mm.fontwidth,1*mm.fontheight, str$(SN!(2,1),3,1) ' inside temperature
  sDT=date$+time$
  s="  "+mid$(sDT,4,2)+"/"+mid$(sDT,1,2)+"/"+mid$(sDT,9,2)+"  "+mid$(sDT,11)+" "
  text 54*mm.fontwidth,1*mm.fontheight, s ' inside temperature
  
  ln=9
  text 9*mm.fontwidth,ln*mm.fontheight, str$(SN!(3,1),3,1) ' yard feed
  text 20*mm.fontwidth,ln*mm.fontheight, str$(SN!(5,1),3,1) ' tank feed
  text 32*mm.fontwidth,ln*mm.fontheight, str$(SN!(8,1),3,1) ' blower feed

  ln=13
  text 26*mm.fontwidth,ln*mm.fontheight, str$(SN!(7,1),3,1) ' tank center
  
  ln=18
  text 9*mm.fontwidth,ln*mm.fontheight, str$(SN!(4,1),3,1) ' yard return
  text 20*mm.fontwidth,ln*mm.fontheight, str$(SN!(6,1),3,1) ' tank return
  text 32*mm.fontwidth,ln*mm.fontheight, str$(SN!(9,1),3,1) ' blower return
  

END SUB 'DISPLAY.SCREEN
'
'     5         6         7              6         7                  
'5678901234567890123456789012345678 5678901234567890123456789012345678
'╦════════════════════════════════╣ ╦════════════════════════════════╣
'║                                ║ ║                                ║
'║   CHANGE INDOOR TEMP SETTINGS  ║ ║   CHANGE TANK TEMP SETTINGS    ║
'║                                ║ ║                                ║
'║                                ║ ║                                ║
'║           HEAT BELOW           ║ ║           HEAT BELOW           ║
'║       NOW ###    NEW ###       ║ ║       NOW ###    NEW ###       ║
'║                                ║ ║                                ║
'║           COOL ABOVE           ║ ║           COOL ABOVE           ║
'║       NOW ###    NEW ###       ║ ║       NOW ###    NEW ###       ║
'║                                ║ ║                                ║
'╩════════════════════════════════╣ ╩════════════════════════════════╣
'         6         7   
' 5678901234567890123456789012345678
' ╦════════════════════════════════╣'08
' ║                                ║'09
' ║  CHANGE OUTDOOR TEMP SETTINGS  ║'10
' ║                                ║'11
' ║                                ║'12
' ║           HEAT BELOW           ║'13
' ║       NOW ###    NEW ###       ║'14
' ║                                ║'15
' ║           COOL ABOVE           ║'16
' ║       NOW ###    NEW ###       ║'17
' ║                                ║'18
' ╩════════════════════════════════╣'19
'
'          1         2         3         4         5         6         7   
'0123456789012345678901234567890123456789012345678901234567890123456789012345678
'║   ┌────┐┌────┐┌────┐┌────┐    ┌────┐┌────┐    ┌───────┐┌───────┐┌───────┐   ║'04
'║   │<0> ││<1> ││<2> ││<3> │    │<4> ││<5> │    │<6>    ││ <7>   ││<8>    │   ║'05
'║   │OFF ││AUTO││COOL││HEAT│    │AUTO││ON  │    │INDOOR ││ TANK  ││OUTDOOR│   ║'06
'║   └────┘└────┘└────┘└────┘    └────┘└────┘    └───────┘└───────┘└───────┘   ║'07
'
'          1         2         3         4         5         6         7   
'0123456789012345678901234567890123456789012345678901234567890123456789012345678
'╔═══════════════╦══════════════╦════════════════════╦═════════════════════════╗'00
'║ OUTDOOR ###.# ║ INDOOR ###.# ║  GEOTHERMAL SYSTEM ║ MM/DD/YYYY  HH:MM:SS AM ║'01
'╠═══════════════╩══════════════╩════════════════════╩═════════════════════════╣'02
'║         SELECT MODE            SET BLOWER          SET TEMPERATURES         ║'03
'║   ┌────┐┌────┐┌────┐┌────┐    ┌────┐┌────┐    ┌───────┐┌───────┐┌───────┐   ║'04
'║   │<0> ││<1> ││<2> ││<3> │    │<4> ││<5> │    │<6>    ││ <7>   ││<8>    │   ║'05
'║   │OFF ││AUTO││COOL││HEAT│    │AUTO││ON  │    │INDOOR ││ TANK  ││OUTDOOR│   ║'06
'║   └────┘└────┘└────┘└────┘    └────┘└────┘    └───────┘└───────┘└───────┘   ║'07
'╠════════════════════════════════════════════╦════════════════════════════════╣'08
'║ ┌─────┐###.#┌────┐###.#┌─────┐###.#┌─────┐ ║                                ║'09
'║ │     ╞══<══╡<<<<╞══<══╡     ╞══<══╡     │ ║                                ║'10
'║ │     │     └────┘     │TANK │     │  B  │ ║                                ║'11
'║ │  Y  │    HEAT PUMP   │     │     │  L  │ ║                                ║'12
'║ │  A  │                │###.#│     │  O  │ ║                                ║'13
'║ │  R  │      EXP       │     │     │  W  │ ║                                ║'14
'║ │  D  │     VALVE      │     │     │  E  │ ║                                ║'15
'║ │     │     ┌────┐     │     │     │  R  │ ║                                ║'16
'║ │     ╞══>══╡>xx>╞══>══╡     ╞══>══╡     │ ║                                ║'17
'║ └─────┘###.#└────┘###.#└─────┘###.#└─────┘ ║                                ║'18
'╠════════════════════════════════════════════╩════════════════════════════════╣'19
'║    ┌───────┐   ┌───────┐   ┌───────┐   ┌───────┐   ┌───────┐   ┌───────┐    ║'20
'║ ─>─┤DISPLAY├─>─┤TMP  ST├─>─┤TMP  RD├─>─┤COMPUTE├─>─┤UP LOAD├─>─┤OUTPUTS├─>─ ║'21
'║    └───────┘   └───────┘   └───────┘   └───────┘   └───────┘   └───────┘    ║'22
'╚═════════════════════════════════════════════════════════════════════════════╝'23
'
'          1         2         3         4         5         6         7   
'0123456789012345678901234567890123456789012345678901234567890123456789012345678
'╠════════════════════════════════════════════╩════════════════════════════════╣'19
'║    ┌───────┐   ┌───────┐   ┌───────┐   ┌───────┐   ┌───────┐   ┌───────┐    ║'20
'║ ─>─┤DISPLAY├─>─┤TMP  ST├─>─┤TMP  RD├─>─┤COMPUTE├─>─┤UP LOAD├─>─┤OUTPUTS├─>─ ║'21
'║    └───────┘   └───────┘   └───────┘   └───────┘   └───────┘   └───────┘    ║'22 
'╚═════════════════════════════════════════════════════════════════════════════╝'23
'
'          1         2         3         4         5         6         7   
'0123456789012345678901234567890123456789012345678901234567890123456789012345678
'╔═══════════════╦══════════════╦════════════════════╦═════════════════════════╗'00
'║ OUTDOOR       ║ INDOOR       ║ GEOTHERMAL  SYSTEM ║                         ║'01
'╠═══════════════╩══════════════╩════════════════════╩═════════════════════════╣'02
'║           SELECT MODE               SET BLOWER         SET TEMPERATURES     ║'03
'║┌──────┐┌──────┐┌──────┐┌──────┐  ┌──────┐┌──────┐   ┌──────┐┌──────┐┌──────┐║'04
'║│  <0> ││ <1>  ││ <2>  ││ <3>  │  │ <4>  ││ <5>  │   │<6>   ││ <7>  ││<8>   │║'05
'║│  OFF ││ AUTO ││ COOL ││ HEAT │  │ AUTO ││ ON   │   │INDOOR││ TANK ││OUT DR│║'06
'║└──────┘└──────┘└──────┘└──────┘  └──────┘└──────┘   └──────┘└──────┘└──────┘║'07
'╠════════════════════════════════════════════╦════════════════════════════════╣'08
'║ ┌─────┐     ┌────┐     ┌─────┐     ┌─────┐ ║                                ║'09
'║ │     ╞══<══╡<<<<╞══<══╡     ╞══<══╡     │ ║                                ║'10
'║ │     │     └────┘     │TANK │     │  B  │ ║                                ║'11
'║ │  Y  │    HEAT PUMP   │     │     │  L  │ ║                                ║'12
'║ │  A  │                │     │     │  O  │ ║                                ║'13
'║ │  R  │      EXP       │     │     │  W  │ ║                                ║'14
'║ │  D  │     VALVE      │     │     │  E  │ ║                                ║'15
'║ │     │     ┌────┐     │     │     │  R  │ ║                                ║'16
'║ │     ╞══>══╡>xx>╞══>══╡     ╞══>══╡     │ ║                                ║'17
'║ └─────┘     └────┘     └─────┘     └─────┘ ║                                ║'18
'╠════════════════════════════════════════════╩════════════════════════════════╣'19
'║                                                                             ║'20
'║                                                                             ║'21
'║                                                                             ║'22
'╚═════════════════════════════════════════════════════════════════════════════╝'23
'
' 201  203  187  205    218  194  191  196     209  199
'  ╔    ╦    ╗    ═      ┌    ┬    ┐    ─       ╤    ╟
' 204  206  185  186    195  197  180  179     207  182
'  ╠    ╬    ╣    ║      ├    ┼    ┤    │       ╧    ╢
' 200  202  188         192  193   217
'  ╚    ╩    ╝           └    ┴     ┘
'

SUB DISPLAY.MENU
'MENU
' display.menu : M.key$=(O│D|H|A|N|U|1|2|3)
'
' IF menukey THEN set M.key
'
'display.menu
'
'if user selects MODE
' OFF THEN stop heat pump & furnace blower & furnace burner & yard loop pump & tank loop pump
' BLWR  THEN set furnace blower on
' AUTO  THEN IF outside > changeover THEN COOL ELSE HEAT
' HEAT  THEN IF hall set > hall temp THEN set furnace burner on & furnace blower on
' COOL  THEN IF tank set < tank temp
'     THEN  set heat pump ON - COOL & yard loop ON & tank loop ON
'     ELSE  set heat pump off & yard loop OFF & tank loop OFF
'   ELSE IF hall set < hall temp
'     THEN set blower loop pump ON & blower ON
'     else set blower loop pump OFF & blower OFF

END SUB DISPLAY.MENU
'
'==============
'The user can control the following:
'    a) MODE off
'    b) MODE cool
'    c) MODE heat
'    d) MODE auto,
'    e) BLOWER auto/continuous
'    f) desired heat/cool automatic changeover set point
'    g) desired hall set point
'    h) desired tank set point

'================
'The menu would trap a valid choice and return the choice in the var t%.

read setpoint log file and restore setpoints

SUB INITIALIZE ' e.g., read display template
  cls
  open "\dl\micromite\_mmb4w\geo\g9_display.txt" for input as #1
  do while not EOF(#1): inc nLines: line input #1, sLine(nLines): print sLine(nlines): loop
  close #1
  text 0,mm.fontheight*29,"" ' set cursor
'  print nLines;" read; Temp Sensors: ";
  for i=1 to nSensors
'      print SN!(i,2);"-";SN!(i,1);" ";
  next i
  fh=mm.fontheight
END SUB ' INITIALIZE

SUB sensor.read
  if SN!(1,1)=999 then ' not yet initialized
    sCmd1="s="
    sCMD3=""
    for i=1 to nSensors
     sCmd1=sCmd1+"str$(tempr("+str$(fix(SN!(i,2)))+"))+"+QT+" "+QT+"+"
     sCmd3=sCmd3+"TEMPR START "+str$(fix(SN!(i,2)))+",3:"
    next i
    sCmd1=mid$(sCmd1,1,len(sCmd1)-1)
    text 0,26*fh,""
    print mid$(sCmd1,1,100)
    if len(sCmd1)>100 then print mid$(sCmd1,101)
    sCmd2="print "+QT+"! "+QT+"+s"
    print sCmd2
    print mid$(sCmd3,1,100)
    if len(sCmd3)>100 then print mid$(sCmd3,101)
    open "com4:115200" as #1
    pause 20
    if loc(#1) then: a=input$(255,#1): do while len(a): a=input$(255,#1): loop: endif ' clear
    print #1,"new"
    pause 1500
    print #1,"option display 70,240"
    pause 1000
    print #1,"dim string s"
    pause 10
    SN!(1,1)=888
  endif
  text 0,32*fh,"" ' position for log of returned lines
  print #1,sCmd1
  pause 30
  print #1,sCmd2
  pause 30
  print #1,sCmd3
  s=""
  for j=1 to 20 ' try to get returned data for up to one second
    if loc(#1) then
      pause 100 ' allow all data to arrive
      do while loc(#1) > 0
        line input #1,a
        print ": ",a
        if mid$(a,1,1)="!") then
          print a
          for i=2 to 11: f=val(field$(a,i," "))/5*9+32: s=s+str$(f,3,4)+" ": SN!(i-1,1)=f: next i
'          for i=12 to 13: j=val(field$(a,i," ")): s=s+str$(j)+" ": next i
          print ":: "+s
          a=input$(255,#1): do while len(a): a=input$(255,#1): loop ' clear response line
          exit do
        endif
      loop
    endif
    if s<>"" then exit for ' we have gotten data back
    pause 50
  next j
END SUB ' sensor.read